unit PSPBase;

interface

uses classes;

type
TPSPWriteEvent = procedure (Sender:TObject;Data:String) of object;
TPSPReadEvent = procedure (Sender:TObject;var Data:String) of object;

TPSPServer = class(TComponent) 
 private
	fvars:tstringlist;
 public
     constructor create(aowner:tcomponent);override;
     destructor destroy;override;
     procedure Clear;
     procedure SetVar (name,value:String);
 published
     function Variables(index:integer):String;
     function VariableCount:integer;
     function IndexOf (name:String)	:integer;
 end;

TPSPOutputstream = class(TComponent)
  private
        fOnWrite:TPSPWriteevent;
  published
      property OnWrite:TPSPWriteevent read fonwrite write fonwrite;
      procedure Write (Data:STring);  virtual;
  end;

{PSP Execution Formats:}
TPSPFormat=INTEGER;

const
{
I will break each component down into the following categories:
the nukmbers will be an integer that can be broken down by powers
of 2: 0,1,2,4,8,16, etc. so you can get
the interface information
MINIMUM-PSP: handle PSP_INTERFACE,PSP_HTML
SERVICE-PSP: handle PSP_INTERFACE,PSP_UNATTENDED
FULL-PSP: as minimum + XML
FULL_SERVICE_PSP: as minimum + XML
I am not reequiring the component stream yet.
by version 1.0 there will be the same 4 with the addition
of [xxx]_PSP_RTTI ... adding component streaming. streaming includes
all children.
}
MINIMUM_PSP = 2;
MINIMUM_SERVICE_PSP = 1;
FULL_PSP = 6;
FULL_SERVICE_PSP = 5;

{Break down}
PSP_INTERFACE = 0   ;
{Your component should return the webcomponent interface, listing methods,
 events, etc... basically the PSP FILE minus the executable code.
 THIS WILL BE AUTOMATICALLY IMPLEMENTED!}
PSP_UNATTENDED = 1  ;
{This case is for compnents that are baiscally JUST services, meaning they have some sort of
execution loop, but don't produce content}
PSP_HTML = 2   ;
{well, duh.}
PSP_XML = 4    ;
{same as above, but should return xml}
PSP_COMPONENT_STREAM = 8  ;
{The component is required to build and transmit a text stream
representing its RTTI information
 THIS WILL BE AUTOMATICALLY IMPLEMENTED! by version 1.0
}



type

TPSPObject = class(TComponent)
 private
  fonstop,fonstart,fonpause,fonresume:TNotifyevent;
  fonread:TPSPReadevent;
  fonwrite : TPSPWriteevent;
  foutputstream:TPSPOutputstream;
  fserver :TPSPServer;
  fispaused:Boolean;
  fisstarted:Boolean;
  fformat:TPSPFormat;
 public
   constructor create (Aowner:TComponent);   override;
   destructor destroy;override;
   procedure Start;
   procedure Stop;
   procedure Pause;
   procedure Resume;
   procedure SetServer(NewServer:TPSPServer);
   procedure SetOutputStream (OS:TPSPOutputstream);

   property OnWrite:TPSPWriteEvent read fonwrite write fonwrite;
   procedure Write (Data:String);
   procedure WriteLn (Data:String);

   {reserved for later use:}
   property OnRead:TPSPReadEvent read fonread write fonread;
   function Read  :String;
   function ReadLn  :String;
   {end reserved block}
{These may be overriden by PSP Methods}
   procedure OnExecute;virtual;
   procedure OnStart;virtual;
   procedure OnStop;virtual;
   procedure OnPause;virtual;
   procedure OnResume;virtual;
   procedure OnCreate;virtual;
   procedure OnDestroy; virtual;
 published
   property Format:TPSPFormat read fformat;
   property IsPaused:boolean read fispaused write fispaused;
   property IsStarted:boolean read fisstarted write fisstarted;
   procedure Execute;

   property Outputstream:TPSPOutputstream read fOutputstream write SetOutputstream;
   property Server:TPSPServer read fserver write SetServer;   

end;

TPSPList = class(TComponent)
 private
  fList:TList;
 public
  constructor create (Aowner:TComponent);override;
  destructor destroy;override;
  function Add (Obj:TPSPObject):integer;
  procedure Delete(index:integer);
 published
  function Items(index:integer):TPSPObject;
  function Count:integer;
end;

implementation

procedure TPSPObject.OnExecute;
begin {} end;
procedure TPSPObject.OnStart;
begin {} end;
procedure TPSPObject.OnStop;
begin {} end;
procedure TPSPObject.OnPause;
begin {} end;
procedure TPSPObject.OnResume;
begin {} end;
procedure TPSPObject.OnCreate;
begin {} end;
procedure TPSPObject.OnDestroy; 
begin {} end;

constructor TPSPObject.create (Aowner:TComponent);
   begin
     inherited create(aowner);
     OnCreate;
   end;
destructor TPSPObject.destroy;
begin
   try
      OnDestroy; 
   finally
       inherited destroy;
   end;
end;
procedure TPSPObject.Start;     
   begin
OnSTart;
   end;
procedure TPSPObject.Stop;     
   begin
OnStop;
   end;
procedure TPSPObject.Pause;     
   begin
OnPause;
   end;
procedure TPSPObject.Resume;     
   begin
OnResume;
   end;

procedure TPSPObject.Execute;     
   begin
 OnExecute;
   end;

procedure TPSPObject.Write  (Data:String);  
   begin
     if assigned(foutputstream) then foutputstream.write(Data);
   end;
procedure TPSPObject.WriteLn (Data:String);  
   begin
     if assigned(foutputstream) then foutputstream.write(Data+#13+#10);
   end;
function TPSPObject.Read  :String;        
   begin
{Not implemented for v.9}
   end;
function TPSPObject.ReadLn  :String;        
   begin
{Not implemented for v.9}
   end;

procedure TPSPObject.SetServer(NewServer:TPSPServer);   
var i:integer; psp:TPSPObject;
   begin
   fServer:=NewServer;
   for i:= 0 to self.ComponentCount -1 do
     begin
         psp:=TPSPObject(self.components[i]);
         psp.SetServer(NewServer);
     end;
   end;
procedure TPSPObject.SetOutputStream (OS:TPSPOutputstream);  
var i:integer; psp:TPSPObject;
   begin
 fOutputstream := os;
   for i:= 0 to self.ComponentCount -1 do
     begin
         psp:=TPSPObject(self.components[i]);
         psp.SetOutputStream(os);
     end;
   end;

procedure TPSPOutputstream.Write (Data:STring);
begin
  if assigned(fonwrite) then fonwrite(self,data);
end;

constructor TPSPServer.create(aowner:tcomponent); 
begin
 inherited create(aowner);
 fvars:=tstringlist.create;
end;
destructor TPSPServer.destroy; 
begin
try
  fvars.clear;
  fvars.free;
finally
  inherited destroy;
end;
end;
procedure TPSPServer.Clear;
begin
fvars.clear;
end;

procedure TPSPServer.SetVar (name,value:String);
begin
if fvars.indexof(name+'='+value)=0 then fvars.add(name+'='+value);
end;

function TPSPServer.Variables(index:integer):String;
begin
result:= fvars.names[index];
end;

function TPSPServer.VariableCount:integer;
begin
result:=fvars.count;
end;

function TPSPServer.IndexOf (name:String)	:integer;
var i,epos:integer;h,c:String;
begin
i:=0;
result:=-1;
while i<fvars.count do
 begin
  h:=fvars[i];
  epos:=pos('=',h);
  c:= copy(fvars[i],1,epos-1);
  if c=name then 
     begin
	 result:=i;
       i:=fvars.count;
     end;
  inc(i);
  end;
end;

  constructor TPSPList.create (Aowner:TComponent);
  begin
  inherited create(aowner);
  flist:=tlist.free;
  end;

  destructor TPSPList.destroy;
  var p:TPSPObject;
  begin
   try
    while flist.count>0 do
     begin
      p:=flist[flist.count-1];
      p.pause;
      p.stop;
      p.free;
      flist.delete(flist.count-1);
     end;
     flist.free;
  finally
    inherited destroy;
  end;
  end;

  function TPSPList.Add (Obj:TPSPObject):integer;
  begin
    result:=flist.add(Obj);
  end;
  procedure TPSPList.Delete(index:integer);
  begin
   flist.delete(index);
  end;
  function TPSPList.Items(index:integer):TPSPObject;
  begin
    result:=TPSPObject(flist[index]);
  end;
  function Count:integer;
  begin
  result:=flist.count;
  end;

end.